@@ -23,7 +23,7 @@ if 'phonostats' in settings.INSTALLED_APPS:
     from phonostats.utils import getNeighCount,getPhonotacticProb,guessStress
 
 if 'celex' in settings.INSTALLED_APPS:
-    from celex.utils import categorize_words,get_lexical_info
+    from celex.utils import categorize_words,get_lexical_info,lookupCat
 
 if 'mysql' in settings.DATABASES['default']['ENGINE']:
     from .helper import mysql_ur_string_lookup as UR_LOOKUP
@@ -607,15 +607,21 @@ class WordToken(caching.base.CachingMixin,models.Model):
             prev_context = self.getPreviousContext(window = int(window))
         else:
             prev_context = self.getPreviousContext()
-        catted = categorize_words([str(x)
-                                for x in prev_context
-                                    if str(x) != str(self)]
-                                + [str(self)])
-        cat_word = catted.pop(-1)
-        tagged_context = ['#'.join([x,'1']) for x in
-                            catted
-                            if x[-2] == '#' and x[-1] in ['n','v','a','r']]
-        if cat_word.endswith('NA'):
+        #catted = categorize_words([str(x)
+        #                        for x in prev_context
+        #                            if str(x) != str(self)]
+        #                        + [str(self)])
+
+        #cat_word = catted.pop(-1)
+        #tagged_context = ['#'.join([x,'1']) for x in
+        #                    catted
+        #                    if x[-2] == '#' and x[-1] in ['n','v','a','r']]
+        prev_context = filter(lambda x: x.getCelexCat() in ['N','V','A','ADV'],prev_context)
+        tagged_context = map(lambda x: '#'.join([str(x),
+                                                x.getCelexCat().lower().replace('adv','r'),
+                                                '1']),prev_context)
+        cat_word = '#'.join([str(self),self.getCelexCat(),'1'])
+        if cat_word.endswith('na#1'):
             cat = self.Category.Label
             if 'N' in cat:
                 cat = 'n'
@@ -627,8 +633,8 @@ class WordToken(caching.base.CachingMixin,models.Model):
                 cat = 'r'
             else:
                 cat = 'NA'
-            cat_word  = '#'.join([cat_word.split("#")[0],cat])
-        sp = getSemanticRelatedness(cat_word+"#1",tagged_context,style=style)
+            cat_word  = '#'.join([cat_word.split("#")[0],cat,'1'])
+        sp = getSemanticRelatedness(cat_word,tagged_context,style=style)
         if style == 'A' and window == 'A':
             self.PrevSemPred = sp
             self.save()
@@ -643,15 +649,21 @@ class WordToken(caching.base.CachingMixin,models.Model):
             foll_context = self.getFollowingContext(window = int(window))
         else:
             foll_context = self.getFollowingContext()
-        catted = categorize_words([str(x)
-                                for x in foll_context
-                                    if str(x) != str(self)]
-                                + [str(self)])
-        cat_word = catted.pop(-1)
-        tagged_context = ['#'.join([x,'1']) for x in
-                            catted
-                            if x[-2] == '#' and x[-1] in ['n','v','a','r']]
-        if cat_word.endswith('NA'):
+        #catted = categorize_words([str(x)
+        #                        for x in foll_context
+        #                            if str(x) != str(self)]
+        #                        + [str(self)])
+
+        #cat_word = catted.pop(-1)
+        #tagged_context = ['#'.join([x,'1']) for x in
+        #                    catted
+        #                    if x[-2] == '#' and x[-1] in ['n','v','a','r']]
+        foll_context = filter(lambda x: x.getCelexCat() in ['N','V','A','ADV'],foll_context)
+        tagged_context = map(lambda x: '#'.join([str(x),
+                                                x.getCelexCat().lower().replace('adv','r'),
+                                                '1']),foll_context)
+        cat_word = '#'.join([str(self),self.getCelexCat(),'1'])
+        if cat_word.endswith('na#1'):
             cat = self.Category.Label
             if 'N' in cat:
                 cat = 'n'
@@ -663,8 +675,8 @@ class WordToken(caching.base.CachingMixin,models.Model):
                 cat = 'r'
             else:
                 cat = 'NA'
-            cat_word  = '#'.join([cat_word.split("#")[0],cat])
-        sp = getSemanticRelatedness(cat_word+"#1",tagged_context,style=style)
+            cat_word  = '#'.join([cat_word.split("#")[0],cat,'1'])
+        sp = getSemanticRelatedness(cat_word,tagged_context,style=style)
         if style == 'A' and window == 'A':
             self.FollSemPred = sp
             self.save()
@@ -953,6 +965,13 @@ class WordToken(caching.base.CachingMixin,models.Model):
             return False
         return True
 
+    def getCelexCat(self):
+        if 'CelexCategory' not in self.Output:
+            cat = lookupCat(self.WordType.Label)
+            self.Output['CelexCategory'] = cat
+            self.save()
+        return self.Output['CelexCategory']
+
     def getAnalysisLines(self,output_form):
         form = output_form.cleaned_data
         wanted_fields = output_form.get_wanted_fields()
@@ -995,20 +1014,18 @@ class WordToken(caching.base.CachingMixin,models.Model):
                     outline['wasRepeated'] = 'True'
                 else:
                     outline['wasRepeated'] = 'False'
+        if 'CelexCategory' not in outline:
+            cat = lookupCat(self.WordType.Label)
+            outline['CelexCategory'] = cat
+        if outline['CelexCategory'] not in ['N','ADV','A','V']:
+            return None
 
-        celex_info = get_lexical_info(self.WordType.Label)
+        if 'C' in form['lexical_scale']:
+            celex_info = get_lexical_info(self.WordType.Label)
 
         if 'B' in form['category']:
             if 'BuckeyeCategory' not in outline:
                 outline['BuckeyeCategory'] = self.Category.Label
-        if 'C' in form['category']:
-            if 'CelexCategory' not in outline:
-                try:
-                    outline['CelexCategory'] = celex_info['Cat']
-                except KeyError:
-                    outline['CelexCategory'] = 'NA'
-        if outline['CelexCategory'] not in ['N','ADV','A','V']:
-            return None
 
         if form['wordDuration']:
             if 'WordDuration' not in outline:
